//
// Created by trotFunky on 02/11/2019.
//

#ifndef TESTS_OPENGL_ENGINE_H
#define TESTS_OPENGL_ENGINE_H

#include <GL/glut.h>
#include <GL/gl.h>

#include "InputStatus.h"
#include "Camera.h"

/// OpenGL Engine
namespace OGLE
{
    /// Display function provided at setup.
    void (*custom_display)();
    static double aspect_ratio;
    static Camera camera({10,0,10},{-10,0,-10},{0,1,0});

    void reshape(int new_x, int new_y)
    {
        glViewport(0,0,new_x,new_y);

        aspect_ratio = (double)new_x/new_y;
        InputStatus::window_size.x = new_x;
        InputStatus::window_size.y = new_y;
    }

    /// Used for glutDisplay. Wraps the custom_display function with the required OpenGL calls.
    void display_wrapper()
    {
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glMatrixMode(GL_PROJECTION);
        // Prepare view
        glLoadIdentity();
        gluPerspective(60,OGLE::aspect_ratio,0.1,30000);

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        OGLE::camera.look();

        custom_display();

        glutSwapBuffers();
    }

    void setup(int& argc, char** argv, void (*display_callback)(), const std::string& window_name = "OpenGL custom engine")
    {
        glutInit(&argc,argv);
        // Generate window with GLUT
        glutInitDisplayMode(GLUT_RGB| GLUT_DEPTH | GLUT_DOUBLE);
        glutCreateWindow(window_name.c_str());
        glutIgnoreKeyRepeat(true);
        glutSetCursor(GLUT_CURSOR_NONE);

        // Init OpenGL
        glClearColor(0,0,0,1);
        glClearDepth(1.0);
        glEnable(GL_DEPTH_TEST);

        // Setup OPenGL to use textures
        glEnable(GL_TEXTURE_2D);
        glAlphaFunc(GL_GREATER, 0.5);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT/GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT/GL_CLAMP);
        glTexEnvi(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATE);

        glViewport(0,0,glutGet(GLUT_WINDOW_WIDTH),glutGet(GLUT_WINDOW_HEIGHT));

        custom_display = display_callback;

        glutDisplayFunc(display_wrapper);
        glutIdleFunc(display_wrapper);
        glutReshapeFunc(reshape);
        InputStatus::register_glut_callbacks();
    }
}

#endif //TESTS_OPENGL_ENGINE_H
